package jp.sf.amateras.javalib.dbunit;

import java.io.FileInputStream;

import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.QueryDataSet;
import org.dbunit.dataset.CompositeDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.ext.oracle.OracleDataTypeFactory;
import org.dbunit.operation.DatabaseOperation;

public class CompositeDataSetTest extends DBTestCase {

	public CompositeDataSetTest(String name) {
		super(name);
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS,
				"org.postgresql.Driver");
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL,
				"jdbc:postgresql://localhost:5432/mydb");
		// System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA,
		// "public" );
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME,
				"user");
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD,
				"password");
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception {
		return DatabaseOperation.REFRESH;
	}

	@Override
	protected DatabaseOperation getTearDownOperation() throws Exception {
		return DatabaseOperation.NONE;
	}

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
	 * Override method to set custom properties/features
	 */
	@Override
	protected void setUpDatabaseConfig(DatabaseConfig config) {
		config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
		// Oracleの場合
		 config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new
		 OracleDataTypeFactory());
	}

	/**
	 * データセットの複合/クエリデータセット
	 */
	public void testCompsiteDataSet() throws Exception {
		IDataSet beforeDataSet = new FlatXmlDataSetBuilder()
				.build(new FileInputStream("data/init.xml"));
		IDataSet insertDataSet = new FlatXmlDataSetBuilder()
				.build(new FileInputStream("data/insert.xml"));

		// 複合データセット
		@SuppressWarnings("unused")
		CompositeDataSet expectedDataSet = new CompositeDataSet(beforeDataSet,
				insertDataSet);

		// クエリデータセット
		QueryDataSet dataSet = new QueryDataSet(getConnection());
		dataSet.addTable("FOO", "SELECT * FROM employee WHERE name = 'taro'");
		dataSet.addTable("book");
	}
}
