package jp.sf.amateras.javalib.dbunit;

import java.io.FileInputStream;
import java.io.FileOutputStream;

import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatDtdDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.dataset.xml.FlatXmlWriter;
import org.dbunit.ext.oracle.OracleDataTypeFactory;
import org.dbunit.operation.DatabaseOperation;

public class DBUnitTest extends DBTestCase {

	public DBUnitTest(String name) {
		super(name);
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS,
				"org.postgresql.Driver");
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL,
				"jdbc:postgresql://localhost:5432/mydb");
		// System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA,
		// "public" );
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME,
				"user");
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD,
				"password");
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception {
		// リフレッシュする
		return DatabaseOperation.REFRESH;
	}

	@Override
	protected DatabaseOperation getTearDownOperation() throws Exception {
		// 何もしない
		return DatabaseOperation.NONE;
	}

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値データを返す
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
	 * DTDを出力する。
	 */
	public void testPutFlatDtdDataSet() throws Exception {
		FlatDtdDataSet.write(
			    getConnection().createDataSet(),
			    new FileOutputStream("data/datasetDtd.xml"));
	}

	public void testPutFlatXmlDataWithDtd() throws Exception {
		IDataSet dataSet = getConnection().createDataSet();
		FlatXmlWriter datasetWriter = new FlatXmlWriter(
			       new FileOutputStream("data/dataset.xml"));
		datasetWriter.setDocType("data/datasetDtd.xml");
		datasetWriter.write(dataSet);
	}

	/**
	 * Override method to set custom properties/features
	 */
	@Override
	protected void setUpDatabaseConfig(DatabaseConfig config) {
		// Oracleの場合
		config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY,
				new OracleDataTypeFactory());
	}

}
