package jp.sf.amateras.javalib.dbunit;

import java.io.File;

import org.dbunit.Assertion;
import org.dbunit.IDatabaseTester;
import org.dbunit.JdbcDatabaseTester;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class DBUnitTestSample {

	private IDatabaseTester databaseTester;

	public DBUnitTestSample() throws Exception{
		databaseTester = new JdbcDatabaseTester("org.postgresql.Driver",
				"jdbc:postgresql://localhost:5432/mydb", "user", "password");
	}

	@Before
	public void before() throws Exception {
		// 初期化用のデータセット定義
        IDataSet dataSet =
        	new FlatXmlDataSetBuilder().build(new File("data/init.xml"));

        databaseTester.setDataSet( dataSet );
        databaseTester.setSetUpOperation(DatabaseOperation.REFRESH);
        // will call default setUpOperation
        databaseTester.onSetup();
	}

	@After
	public void after() throws Exception {
		// will call default tearDownOperation
        databaseTester.setTearDownOperation(DatabaseOperation.NONE);
		databaseTester.onTearDown();
	}

	/**
	 * テーブル全体の内容を比較
	 */
	@Test
	public void compareTable() throws Exception {

		// 実テーブルの値を取得
		IDataSet databaseDataSet = databaseTester.getConnection().createDataSet();
		ITable actualTable = databaseDataSet.getTable("employee"); // テーブル名を指定

		// 期待値をXMLデータから取得
		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expected.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee"); // テーブル名を指定

		// アサーション実行
		Assertion.assertEquals(expectedTable, actualTable);

	}

	@Test
	public void compareTable2() throws Exception {

		// 実テーブルの値を取得
		IDataSet databaseDataSet = databaseTester.getConnection().createDataSet();
		ITable actualTable = databaseDataSet.getTable("employee"); // テーブル名を指定

		// 期待値をXMLデータから取得
		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expected.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee"); // テーブル名を指定

		// アサーション実行
		Assertion.assertEquals(expectedTable, actualTable);

	}
}
