package jp.sf.amateras.javalib.dbunit;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;

import org.dbunit.Assertion;
import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.assertion.DiffCollectingFailureHandler;
import org.dbunit.assertion.Difference;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;

public class DiffCollectingTest extends DBTestCase {

	public DiffCollectingTest(String name) {
		 super( name );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS, "org.postgresql.Driver" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL, "jdbc:postgresql://localhost:5432/mydb" );
//	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA, "public" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME, "user" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD, "passwod" );
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception
    {
        return DatabaseOperation.REFRESH;
    }

	@Override
    protected DatabaseOperation getTearDownOperation() throws Exception
    {
        return DatabaseOperation.NONE;
    }

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
     * Override method to set custom properties/features
     */
	@Override
    protected void setUpDatabaseConfig(DatabaseConfig config) {
        config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
        // Oracleの場合
//        config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new OracleDataTypeFactory());
    }

	/**
	 * 異なる点を確認する
	 */
	public void testDiffHandler() throws Exception {

		IDataSet dataSet = getDataSet();
		ITable actualTable = dataSet.getTable("employee");

		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expectedDiff.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee");

		// DiffCollectingFailureHandlerを作成
		DiffCollectingFailureHandler myHandler = new DiffCollectingFailureHandler();

		// ハンドラを指定してアサーション
		Assertion.assertEquals(expectedTable, actualTable, myHandler);

		// 異なる点が確認できる
		@SuppressWarnings("rawtypes")
		List diffList = myHandler.getDiffList();
		for (int i = 0; i < diffList.size(); i++) {
			Difference diff = (Difference) diffList.get(i);
			System.out.println(diff.getActualValue());
			System.out.println(diff.getExpectedValue());
		}
	}
}
