package jp.sf.amateras.javalib.dbunit;

import java.io.File;
import java.io.FileInputStream;

import org.dbunit.Assertion;
import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.filter.DefaultColumnFilter;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;

public class FilteringTest extends DBTestCase {

	public FilteringTest(String name) {
		 super( name );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS, "org.postgresql.Driver" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL, "jdbc:postgresql://localhost:5432/mydb" );
//	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA, "public" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME, "user" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD, "password" );
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception
    {
        return DatabaseOperation.REFRESH;
    }

	@Override
    protected DatabaseOperation getTearDownOperation() throws Exception
    {
        return DatabaseOperation.NONE;
    }

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
     * Override method to set custom properties/features
     */
	@Override
    protected void setUpDatabaseConfig(DatabaseConfig config) {
        config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
        // Oracleの場合
//        config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new OracleDataTypeFactory());
    }

	/**
	 * 検証に利用するカラムを指定する
	 */
	public void testIncludedColumns() throws Exception {

		// 実テーブルの値を取得
		IDataSet databaseDataSet = getConnection().createDataSet();
		ITable actualTable = databaseDataSet.getTable("employee");

		// 期待値をXMLデータから取得
		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expected.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee");

		// 実テーブル値から期待値に設定されているカラムのみを抽出する
		ITable filteredActualTable = DefaultColumnFilter.includedColumnsTable(
				actualTable, expectedTable.getTableMetaData().getColumns());

		// アサーション実行
		Assertion.assertEquals(expectedTable, filteredActualTable);
	}

	/**
	 * 除外するカラムを指定して比較
	 */
	public void testExcludedColumns() throws Exception {
		// 実テーブルの値を取得
		IDataSet databaseDataSet = getConnection().createDataSet();
		ITable actualTable = databaseDataSet.getTable("employee");

		// 期待値をXMLデータから取得
		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expected.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee");

		// 検証除外カラムの定義
		String[] excludedColumns = { "note" };
		// 実テーブル値から、除外カラムを除く
		ITable filteredActualTable = DefaultColumnFilter.excludedColumnsTable(
				actualTable, excludedColumns);

		// 期待値から、除外カラムを除く
		ITable filteredExpectedTable = DefaultColumnFilter.excludedColumnsTable(
				expectedTable, excludedColumns);

		// アサーション実行
		Assertion.assertEquals(filteredExpectedTable, filteredActualTable);
	}

}
