package jp.sf.amateras.javalib.dbunit;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatDtdDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.dataset.xml.FlatXmlWriter;
import org.dbunit.operation.DatabaseOperation;

public class FlatXmlDataSetTest extends DBTestCase {

	public FlatXmlDataSetTest(String name) {
		super(name);
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS,
				"org.postgresql.Driver");
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL,
				"jdbc:postgresql://localhost:5432/mydb");
		// System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA,
		// "public" );
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME,
				"user");
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD,
				"password");
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception {
		return DatabaseOperation.REFRESH;
	}

	@Override
	protected DatabaseOperation getTearDownOperation() throws Exception {
		return DatabaseOperation.NONE;
	}

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
	 * Override method to set custom properties/features
	 */
	@Override
	protected void setUpDatabaseConfig(DatabaseConfig config) {
		config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
		// Oracleの場合
		// config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new
		// OracleDataTypeFactory());
	}

	/**
	 * フラットXML形式のデータ読み込み
	 */
	public void importFlatXML() throws Exception {

		// XMLデータを取得
		@SuppressWarnings("unused")
		FlatXmlDataSet dataSet = new FlatXmlDataSetBuilder().build(
				new File("data/replace.xml"));

	}

	/**
	 * フラットXML形式でデータセットの内容を出力する。
	 */
	public void testPutFlatXmlDataSet() throws Exception {
		// 実テーブルの値を取得
		IDataSet databaseDataSet = getConnection().createDataSet();
		// フラットXML形式で実テーブルの値を出力する。
		FlatXmlDataSet.write(databaseDataSet, new FileOutputStream(
				"data/datasetFlat.xml"));
	}

	/**
	 * DTDを出力する。
	 */
	public void testPutFlatDtdDataSet() throws Exception {
		FlatDtdDataSet.write(
			    getConnection().createDataSet(),
			    new FileOutputStream("data/datasetDtd.xml"));
	}

	public void testPutFlatXmlDataWithDtd() throws Exception {
		IDataSet dataSet = getConnection().createDataSet();
		FlatXmlWriter datasetWriter = new FlatXmlWriter(
			       new FileOutputStream("data/dataset.xml"));
		datasetWriter.setDocType("data/datasetDtd.xml");
		datasetWriter.write(dataSet);
	}

}
