package jp.sf.amateras.javalib.dbunit;

import java.io.FileInputStream;
import java.sql.Timestamp;

import org.dbunit.Assertion;
import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.ReplacementDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;

public class ReplaceStringTest extends DBTestCase {

	public ReplaceStringTest(String name) {
		 super( name );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS, "org.postgresql.Driver" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL, "jdbc:postgresql://localhost:5432/mydb" );
//	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA, "public" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME, "user" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD, "password" );
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception
    {
        return DatabaseOperation.REFRESH;
    }

	@Override
    protected DatabaseOperation getTearDownOperation() throws Exception
    {
        return DatabaseOperation.NONE;
    }

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
     * Override method to set custom properties/features
     */
	@Override
    protected void setUpDatabaseConfig(DatabaseConfig config) {
        config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
        // Oracleの場合
//        config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new OracleDataTypeFactory());
    }

	/**
	 * 置換文字を利用したデータセット
	 */
	public void testReplacementDataset() throws Exception {

		// 実テーブルの値を取得
		IDataSet databaseDataSet = getConnection().createDataSet();
		ITable actualTable = databaseDataSet.getTable("employee");

		FlatXmlDataSet expectedDataSet = new FlatXmlDataSetBuilder()
				.build(new FileInputStream("data/replace.xml"));

		ReplacementDataSet dataSet = new ReplacementDataSet(expectedDataSet);
		// "[SYSDATE]"を現在時刻に置換するよう設定する
		dataSet.addReplacementObject("[SYSDATE]", new Timestamp(System.currentTimeMillis()));
		// "[NULL]"をnullに置換するよう設定する
		dataSet.addReplacementObject("[NULL]", null);

		ITable expectedTable = dataSet.getTable("employee");

		// アサーション実行
		Assertion.assertEquals(expectedTable, actualTable);

	}
}
