package jp.sf.amateras.javalib.dbunit;

import java.io.File;
import java.io.FileInputStream;

import org.dbunit.Assertion;
import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.SortedTable;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;

public class SortedTableTest extends DBTestCase {

	public SortedTableTest(String name) {
		 super( name );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS, "org.postgresql.Driver" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL, "jdbc:postgresql://localhost:5432/mydb" );
//	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA, "public" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME, "user" );
	        System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD, "password" );
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception
    {
        return DatabaseOperation.REFRESH;
    }

	@Override
    protected DatabaseOperation getTearDownOperation() throws Exception
    {
        return DatabaseOperation.NONE;
    }

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
     * Override method to set custom properties/features
     */
	@Override
    protected void setUpDatabaseConfig(DatabaseConfig config) {
        config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
        // Oracleの場合
//        config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new OracleDataTypeFactory());
    }

	/**
	 * ソートして比較
	 */
	public void testSortedTable() throws Exception {
		// 実テーブルの値を取得
		IDataSet databaseDataSet = getConnection().createDataSet();
		ITable actualTable = databaseDataSet.getTable("employee");

		// 期待値をXMLデータから取得
		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expectedSort.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee");

		// 期待値テーブルのメタデータのカラムソート順で比較する。
		Assertion.assertEquals(new SortedTable(expectedTable), new SortedTable(
				actualTable, expectedTable.getTableMetaData()));
	}

	/**
	 * ソートして比較
	 */
	public void testSortedTable2() throws Exception {
		ITable actualTable = getConnection().createQueryTable("HOGE",
				"SELECT * FROM employee");
		System.out.println(actualTable.getRowCount());

		IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new File(
				"data/expectedSort.xml"));
		ITable expectedTable = expectedDataSet.getTable("employee");

		SortedTable sortedTable1 = new SortedTable(actualTable, new String[] { "name" });
		sortedTable1.setUseComparable(true); // must be invoked immediately after the constructor
		SortedTable sortedTable2 = new SortedTable(expectedTable, new String[] { "name" });
		sortedTable2.setUseComparable(true); // must be invoked immediately after the constructor
		Assertion.assertEquals(sortedTable1, sortedTable2);

	}

}
