package jp.sf.amateras.javalib.dbunit;

import java.io.FileInputStream;
import java.io.FileOutputStream;

import org.dbunit.DBTestCase;
import org.dbunit.PropertiesBasedJdbcDatabaseTester;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.excel.XlsDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;

public class XlsDataSetTest extends DBTestCase {

	public XlsDataSetTest(String name) {
		super(name);
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_DRIVER_CLASS,
				"org.postgresql.Driver");
		System.setProperty(
				PropertiesBasedJdbcDatabaseTester.DBUNIT_CONNECTION_URL,
				"jdbc:postgresql://localhost:5432/mydb");
		// System.setProperty( PropertiesBasedJdbcDatabaseTester.DBUNIT_SCHEMA,
		// "public" );
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_USERNAME,
				"user");
		System.setProperty(PropertiesBasedJdbcDatabaseTester.DBUNIT_PASSWORD,
				"password");
	}

	@Override
	protected DatabaseOperation getSetUpOperation() throws Exception {
		return DatabaseOperation.REFRESH;
	}

	@Override
	protected DatabaseOperation getTearDownOperation() throws Exception {
		return DatabaseOperation.NONE;
	}

	@Override
	protected IDataSet getDataSet() throws Exception {
		// 初期値としてデータを投入
		return new FlatXmlDataSetBuilder().build(new FileInputStream(
				"data/init.xml"));
	}

	/**
	 * Override method to set custom properties/features
	 */
	@Override
	protected void setUpDatabaseConfig(DatabaseConfig config) {
		config.setProperty(DatabaseConfig.PROPERTY_BATCH_SIZE, new Integer(97));
		// Oracleの場合
		// config.setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new
		// OracleDataTypeFactory());
	}

	/**
	 * エクセル形式を入力に利用する。
	 */
	public void testXlsDataset() throws Exception {

		// エクセル形式のファイルを入力に利用する。
		@SuppressWarnings("unused")
		IDataSet dataSet = new XlsDataSet(new FileInputStream("data/data.xls"));

	}

	/**
	 * エクセル形式でデータセットの内容を出力する。
	 */
	public void testPutXlsDataset() throws Exception {

		// 実テーブルの値を取得
		IDataSet databaseDataSet = getConnection().createDataSet();
		// エクセル形式で実テーブルの値を出力する。
		XlsDataSet.write(databaseDataSet, new FileOutputStream("data/data.xls"));
	}
}
