package jp.sf.amateras.javalib.mockrunner.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

public class CustomerDao {
	private static final String URL = "jdbc:hsqldb:hsql://localhost/javalib";
	private static final String USER = "SA";
	private static final String PASSWORD = "";
	private Connection connection;

	public CustomerDao() throws SQLException {
		connection = DriverManager.getConnection(URL, USER, PASSWORD);
	}

	public void close() {
		if (connection != null) {
			try {
				connection.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}

	public void updateByKey(Customer customer) throws SQLException {
		PreparedStatement statement = null;
		try {
			connection.setAutoCommit(false);
			statement = connection.prepareStatement(
					"UPDATE CUSTOMER SET FIRSTNAME = ?, LASTNAME = ?, STREET = ?, CITY = ? WHERE ID = ?");
			statement.setString(1, customer.firstname);
			statement.setString(2, customer.lastname);
			statement.setString(3, customer.street);
			statement.setString(4, customer.city);
			statement.setInt(5, customer.id);

			statement.executeUpdate();
			connection.commit();
		} catch (SQLException e) {
			connection.rollback();
			throw e;
		} finally {
			if(statement != null) {
				try {
					statement.close();
				} catch (SQLException e) {}
			}
		}
	}

	public Customer selectByKey(int key) throws SQLException {
		Customer customer = null;
		PreparedStatement statement = null;
		ResultSet rs = null;
		try {
			statement = connection.prepareStatement("SELECT ID, FIRSTNAME, LASTNAME, STREET, CITY FROM CUSTOMER WHERE ID = ?");
			statement.setInt(1, key);

			rs = statement.executeQuery();

			if (rs.next()) {
				customer = new Customer();
				customer.id = rs.getInt(1);
				customer.firstname = rs.getString(2);
				customer.lastname = rs.getString(3);
				customer.street = rs.getString(4);
				customer.city = rs.getString(5);
			}
		} finally {
			if(rs != null) {
				try {
					rs.close();
				} catch (SQLException e) {}
			}
			if(statement != null) {
				try {
					statement.close();
				} catch (SQLException e) {}
			}
		}
		return customer;
	}

	public static class Customer {
		private int id;
		private String city;
		private String firstname;
		private String lastname;
		private String street;

		public int getId() {
			return id;
		}
		public void setId(int id) {
			this.id = id;
		}
		public String getCity() {
			return city;
		}
		public void setCity(String city) {
			this.city = city;
		}
		public String getFirstname() {
			return firstname;
		}
		public void setFirstname(String firstname) {
			this.firstname = firstname;
		}
		public String getLastname() {
			return lastname;
		}
		public void setLastname(String lastname) {
			this.lastname = lastname;
		}
		public String getStreet() {
			return street;
		}
		public void setStreet(String street) {
			this.street = street;
		}
	}
}
