package jp.sf.amateras.javalib.selenium;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.htmlunit.HtmlUnitDriver;
import org.openqa.selenium.support.ui.Select;

public class WebDriverSample {

	public static void main(String[] args) {
		WebDriverSample example = new WebDriverSample();

		// WebDriverの使用
		example.basicSample();

		// Webページの移動メソッド
		example.navigation();

		// ページ要素へのアクセス
		example.accessElements();

	}

	/**
	 * WebDriverの使用例
	 */
	public void basicSample() {
		// WebDriverの準備　ここではHtmlUnitDriverを利用する。
		WebDriver driver = new HtmlUnitDriver();
		//WebDriver driver = new FirefoxDriver();
		//WebDriver driver = new InternetExplorerDriver();
		//WebDriver driver = new ChromeDriver();

		// サイトにアクセス
		driver.get("http://www.google.com");
		// name属性を利用して要素にアクセスする
		WebElement element = driver.findElement(By.name("q"));
		// 要素に"Test"と入力する
		element.sendKeys("Test");

		// form要素を探し、submitする
		element.submit();

		// ページタイトルを標準出力する
		System.out.println("Page title is: " + driver.getTitle());

	}

	/**
	 * Webページの移動メソッドの利用例
	 */
	public void navigation() {
		// WebDriverの準備　ここではHtmlUnitDriverを利用する。
		WebDriver driver = new HtmlUnitDriver();

		// サイトにアクセス
		driver.get("http://www.google.com");

		// URLを指定して移動 navigate().get()と同じ。
		driver.navigate().to("http://www.google.com");
		// 戻る
		driver.navigate().back();
		// 進む
		driver.navigate().forward();
		// 更新(F5)
		driver.navigate().refresh();
	}

	/**
	 * ページ要素へのアクセス例
	 */
	@SuppressWarnings("unused")
	public void accessElements() {
		// WebDriverの準備　ここではHtmlUnitDriverを利用する。
		WebDriver driver = new HtmlUnitDriver();

		// サイトにアクセス
		driver.get("http://xxx.xxx.xxx");

		// id属性でアクセス
		WebElement element1 = driver.findElement(By.id("userId"));
		// テキストを入力する
		element1.sendKeys("0001");
		// テキストをクリア
		element1.clear();

		// name属性でアクセスして、クリックする
		driver.findElement(By.name("brnOK")).click();

		// XPathでアクセス
		WebElement element3 = driver.findElement(By.xpath("//input[@id='userId']"));

		// Select要素をラベル名で選択する
		Select select = new Select(driver.findElement(By.id("pref")));
		select.selectByVisibleText("沖縄");

	}

}
